<?php
/**
 *  Copyright (C) Prestalia - All Rights Reserved
 *  Unauthorized copying of this file, via any medium is strictly prohibited
 *  Proprietary and confidential
 *
 *  @author    Prestalia <prestalia.it>
 *  @copyright 2019 - 2024 Prestalia
 *  @license   Commercial
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once dirname(__FILE__) . '/../autoload.php';

class SniperfastProductExporter
{
    private $idShop;
    private $idLang;
    private $config;
    private $link;
    private $idSalesCache;
    private $exportFeatures;
    private $exportAttributes;
    private $enabledIdFeatures;
    private $enabledIdAttributeGroups;

    public function __construct($idShop, $idLang, $config, $idSalesCache)
    {
        $this->idShop = $idShop;
        $this->idLang = $idLang;
        $this->config = $config;
        $this->idSalesCache = $idSalesCache;
        $this->link = new Link();

        // check if we have to export features/attributes and build enabled features/attributes array
        $this->exportFeatures = false;
        if ($config['sniper_features'] != '') {
            $this->exportFeatures = true;
            $this->enabledIdFeatures = array_map('intval', explode(',', $config['sniper_features']));
        } elseif ($config['sniper_free_ship'] > 0 || $config['sniper_export_review'] > 0) {
            $this->exportFeatures = true;
        }

        $this->exportAttributes = false;
        if ($config['sniper_attributes'] != '') {
            $this->exportAttributes = true;
            $this->enabledIdAttributeGroups = array_map('intval', explode(',', $config['sniper_attributes']));
        }

        // include file for customer functions
        if (file_exists(dirname(__FILE__) . '/../customerFunctions.php')) {
            require_once dirname(__FILE__) . '/../customerFunctions.php';
        }
    }

    public function exportData($idProduct, &$stats)
    {
        $data = [];
        $skipProduct = false;

        $product = new Product($idProduct, true, $this->idLang, $this->idShop);
        if (!(bool) $product->active || $product->visibility == 'none') {
            ++$stats['skipped_inactive'];
            $skipProduct = true;
        }

        if (!$skipProduct) {
            // Check if product matches price and quantity filter and skip it if it doesn't
            $productQuantity = (int) StockAvailable::getQuantityAvailableByProduct($idProduct);

            if ($this->config['only_in_stock'] && $productQuantity < 1) {
                ++$stats['skipped_quantity'];
                $skipProduct = true;
            }
        }

        if (!$skipProduct) {
            // Assign available CSV fields
            $path = '';
            $currentCategory = 0;
            $categoryPath = '';
            $explodedPath = [];
            // $boost = array();
            if ($currentCategory != (int) $product->id_category_default) {
                $path = Tools::getPath('', $product->id_category_default);
                $path = strip_tags($path);
                $path = str_replace(Configuration::get('PS_NAVIGATION_PIPE'), ' > ', $path);
                $explodedPath = explode(' > ', $path);
                if (count($explodedPath) > 1) {
                    $explodedPath = array_slice($explodedPath, 1);
                    $path = implode(' > ', $explodedPath);
                }
                $categoryPath = trim($path);
                $currentCategory = $product->id_category_default;
            } else {
                $path = $categoryPath;
                $explodedPath = explode(' > ', $path);
            }
            $explodedPath = array_filter(array_map('trim', $explodedPath));

            $data['id_shop'] = $this->idShop;
            $data['id_lang'] = $this->idLang;
            $data['id_product'] = $product->id;
            $data['id_category_default'] = $product->id_category_default;
            $data['reference'] = $product->reference;
            $data['ean13'] = $product->ean13;
            $data['pname'] = $product->name;
            $data['description'] = $product->description;
            $data['description_short'] = $product->description_short;
            $data['supplier_name'] = $product->supplier_name;
            $data['category_link_rewrite'] = Category::getLinkRewrite($product->id_category_default, $this->idLang);
            $data['plink'] = $this->link->getProductLink(
                $product->id,
                $product->link_rewrite,
                $data['category_link_rewrite'],
                $product->ean13,
                $this->idLang,
                $this->idShop
            );
            $data['mname'] = $product->manufacturer_name;
            $data['quantity'] = $productQuantity;
            $supplierInformations =
                Supplier::getProductInformationsBySupplier($product->id_supplier, $product->id, 0);
            $data['supplier_reference'] = isset($supplierInformations['product_supplier_reference']) ? $supplierInformations['product_supplier_reference'] : '';
            $data['cover_image_url'] = $this->getImageLink($product, $this->link, $this->config);

            if (isset($this->idSalesCache[$product->id])) {
                $data['psales'] = (int) $this->idSalesCache[$product->id];
            } else {
                $data['psales'] = 0;
            }
            $data['category'] = implode(' ', $explodedPath);
            for ($i = 1; $i <= 4; ++$i) {
                if (isset($explodedPath[$i - 1])) {
                    $data['category/lvl' . $i] = $explodedPath[$i - 1];
                } else {
                    $data['category/lvl' . $i] = '';
                }
            }

            $decimals = Configuration::get('PS_PRICE_DISPLAY_PRECISION', null, null, $this->idShop);
            if ($decimals === false) {
                $decimals = 2;
            }

            $useTaxes = true;
            if ($this->config['remove_vat']) {
                $useTaxes = false;
            }

            $data['price_deleted'] = (float) round(Product::getPriceStatic(
                $product->id,
                $useTaxes,
                null,
                $decimals,
                null,
                false,
                false,
                1,
                false
            ), 2);
            $data['price_deleted'] = number_format($data['price_deleted'], 2, '.', '');
            $data['price_showed'] = (float) round(Product::getPriceStatic(
                $product->id,
                $useTaxes,
                null,
                $decimals,
                null,
                false,
                true,
                1,
                false
            ), 2);
            $data['price_showed'] = number_format($data['price_showed'], 2, '.', '');

            $validFeatures = [];
            // Export date_add
            $validFeatures[] = [
                'id_feature' => 9980,
                'name' => 'snp_date_add',
                'value' => $product->date_add,
                'position' => 0,
            ];

            // Export date_upd
            $validFeatures[] = [
                'id_feature' => 9979,
                'name' => 'snp_date_upd',
                'value' => $product->date_upd,
                'position' => 0,
            ];

            // Export if product has combinations
            $validFeatures[] = [
                'id_feature' => 9978,
                'name' => 'snp_has_attributes',
                'value' => $product->hasAttributes(),
                'position' => 0,
            ];

            if ($this->exportFeatures || $this->config['available_filter']) {
                $productFeatures = Product::getFrontFeaturesStatic($this->idLang, $data['id_product']);

                if (function_exists('customFeatureAdd')) {
                    $custom_feat = customFeatureAdd($data);
                    if ($custom_feat) {
                        $validFeatures = array_merge($validFeatures, $custom_feat);
                    }
                }

                $freeShippingData = $this->setFreeShipping($this->config['sniper_free_ship'], $data['price_showed']);
                if (!empty($freeShippingData)) {
                    $validFeatures[] = array_shift($freeShippingData);
                }

                foreach ($productFeatures as $productFeature) {
                    if (is_array($this->enabledIdFeatures)
                        && in_array($productFeature['id_feature'], $this->enabledIdFeatures)
                    ) {
                        $validFeatures[] = $productFeature;
                    }
                }
                unset($productFeatures);
                if ($this->config['available_filter']) {
                    if ($data['quantity'] > 0) {
                        $validFeatures[] = [
                            'id_feature' => 9950,
                            'name' => $this->config['available_filter_name'],
                            'value' => $this->config['available_filter_instock'],
                            'position' => 0,
                        ];
                    } else {
                        $validFeatures[] = [
                            'id_feature' => 9950,
                            'name' => $this->config['available_filter_name'],
                            'value' => $this->config['available_filter_outstock'],
                            'position' => 0,
                        ];
                    }
                }
            }
            $data['features'] = json_encode($validFeatures);

            $data['attributes'] = '';
            if ($this->exportAttributes) {
                $productAttributesDetail = Product::getAttributesInformationsByProduct($data['id_product']);
                $validAttributes = [];
                foreach ($productAttributesDetail as $productAttribute) {
                    if (is_array($this->enabledIdAttributeGroups)
                        && in_array($productAttribute['id_attribute_group'], $this->enabledIdAttributeGroups)
                    ) {
                        $validAttributes[] =
                            [
                                'name' => $productAttribute['group'],
                                'value' => $productAttribute['attribute'],
                                'id_attribute_group' => (string) $productAttribute['id_attribute_group'],
                            ];
                    }
                }

                $validAttributesClean = array_unique($validAttributes, SORT_REGULAR);

                if ($this->config['only_in_stock']) {
                    $product = new Product($data['id_product'], true, $this->idLang, $this->idShop);
                    $attributes = $product->getAttributeCombinations($this->idLang);
                    $attributesStock = [];
                    $attributesInStock = [];
                    foreach ($attributes as $a) {
                        $comboQty = 0;
                        if (isset($attributesStock[$a['id_product_attribute']])) {
                            $comboQty = $attributesStock[$a['id_product_attribute']];
                        } else {
                            $comboQty = StockAvailable::getQuantityAvailableByProduct(
                                $a['id_product'],
                                $a['id_product_attribute'],
                                $this->idShop
                            );
                        }
                        if ($comboQty > 0) {
                            if (!isset($attributesInStock[$a['group_name']][$a['attribute_name']])) {
                                $attributesInStock[$a['group_name']][$a['attribute_name']] = 0;
                            }
                            ++$attributesInStock[$a['group_name']][$a['attribute_name']];
                        }
                    }

                    $finalAttributesExport = [];
                    foreach ($validAttributesClean as $vac) {
                        if (isset($attributesInStock[$vac['name']][$vac['value']])) {
                            $finalAttributesExport[] = $vac;
                        }
                    }
                    if (!empty($finalAttributesExport)) {
                        $data['attributes'] = json_encode(array_values($finalAttributesExport));
                    }

                    unset($validAttributesClean);
                    unset($attributes);
                    unset($attributesStock);
                    unset($attributesInStock);
                    unset($finalAttributesExport);
                } else {
                    $data['attributes'] = json_encode(array_values($validAttributesClean));
                }
            }

            // tags
            $product_tags = [];
            $presta_tags = Tag::getProductTags((int) $data['id_product']);
            if (!empty($presta_tags) && isset($presta_tags[$this->idLang])) {
                foreach ($presta_tags[$this->idLang] as $tag_data) {
                    $product_tags[] = $tag_data;
                }
            }

            // export variation reference / ean as additional tags
            if ($product->hasAttributes()) {
                $sql = 'SELECT reference,ean13
                        FROM ' . _DB_PREFIX_ . 'product_attribute
                        WHERE id_product = ' . (int) $product->id;
                $vardata = Db::getInstance()->executeS($sql);
                foreach ($vardata as $att_var) {
                    if (trim($att_var['reference'] != '')) {
                        $product_tags[] = trim($att_var['reference']);
                    }
                    if (trim($att_var['ean13'] != '')) {
                        $product_tags[] = trim($att_var['ean13']);
                    }
                }
            }

            $data['tags'] = json_encode($product_tags);

            if (function_exists('customDataParser')) {
                customDataParser($data);
            }

            $data['alternate_image_url'] = $this->getAlternateImageLink($product, $this->link, $this->config);

            $this->stripTagsFromDesc($data);

            return $data;
        }
    }

    private function stripTagsFromDesc(&$data)
    {
        $allowedTags = '<br><div><span><p><ul><ol><li><b><i><strong><font><h1><h2><h3>' .
            '<table><thead><tbody><tr><th><td>';
        $data['description_short'] = strip_tags($data['description_short'], $allowedTags);
        $data['description'] = strip_tags($data['description'], $allowedTags);
        $data['description_short'] = html_entity_decode($data['description_short'], ENT_COMPAT, 'UTF-8');
        $data['description'] = html_entity_decode($data['description'], ENT_COMPAT, 'UTF-8');
        $data['description_short'] = preg_replace("/\r|\n|\t/", ' ', $data['description_short']);
        $data['description'] = preg_replace("/\r|\n|\t/", ' ', $data['description']);
    }

    private function getImageLink($product, $link, $config)
    {
        $prodImg = Image::getCover($product->id);
        if ($prodImg === false) {
            return $config['base_url'] . '/modules/sniperfast/views/img/notfound.jpg';
        }
        $imgUrl = $link->getImageLink($product->link_rewrite, $prodImg['id_image'], $config['img_type']);
        $this->setImageHttpProtocol($imgUrl, $config['use_https']);

        return $imgUrl;
    }

    private function getAlternateImageLink($product, $link, $config)
    {
        $sql = 'SELECT id_image FROM ' . _DB_PREFIX_ . 'image ' .
                'WHERE cover IS NULL AND id_product = ' . (int) $product->id . ' ORDER BY POSITION ASC LIMIT 1';
        $prodImg = Db::getInstance()->executeS($sql);

        if (empty($prodImg)) {
            return '';
        }

        $imgUrl = $link->getImageLink($product->link_rewrite, $prodImg[0]['id_image'], $config['img_type']);
        $this->setImageHttpProtocol($imgUrl, $config['use_https']);

        return $imgUrl;
    }

    private function setImageHttpProtocol(&$imgUrl, $useHttpsForImages)
    {
        if (Tools::strpos($imgUrl, 'https://') !== false && !$useHttpsForImages) {
            $imgUrl = str_replace('https://', 'http://', $imgUrl);
        } elseif (Tools::strpos($imgUrl, 'http://') !== false && $useHttpsForImages) {
            $imgUrl = str_replace('http://', 'https://', $imgUrl);
        }
        if (Tools::strpos($imgUrl, 'http') === false || Tools::strpos($imgUrl, 'http') > 0) {
            if ($useHttpsForImages) {
                $imgUrl = 'https://' . $imgUrl;
            } else {
                $imgUrl = 'http://' . $imgUrl;
            }
        }
    }

    private function setFreeShipping($freeShippingLimit, $productPrice)
    {
        $tmp = [];
        if (!is_numeric($freeShippingLimit) || $freeShippingLimit <= 0) {
            return $tmp;
        }

        /* Add free shipping */
        $productPrice = (float) $productPrice;
        if ($productPrice >= $freeShippingLimit) {
            $tmp[] = ['id_feature' => 9997, 'name' => 'snp_free_shipping', 'value' => 1, 'position' => 0];
        } else {
            $tmp[] = ['id_feature' => 9997, 'name' => 'snp_free_shipping', 'value' => 0, 'position' => 0];
        }

        return $tmp;
    }

    public function sendData($productsToSend)
    {
        $prod = json_encode($productsToSend);
        // file_put_contents(dirname(__FILE__).'/single_prod.json', $prod);
        // return;

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, CDN_PROTOCOL . '://index.sniperfast.' . CDN_ENDPOINT . '/schedule');
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query([
            'shop' => $this->idShop,
            'lang' => $this->idLang,
            'key' => SniperCommon::getConfig('sniper_key', $this->idShop, $this->idLang),
            'index' => SniperCommon::getConfig('sniper_index_name', $this->idShop, $this->idLang),
            'ip' => $_SERVER['SERVER_ADDR'],
            'products' => $prod,
        ]));

        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $res = curl_exec($ch);
        curl_close($ch);
        $res = json_decode($res, true);

        if (isset($res['status']) && $res['status'] == 'ok') {
            return 1;
        }

        return 0;
    }
}
